<?php

/**
 * This file is part of ILIAS, a powerful learning management system
 * published by ILIAS open source e-Learning e.V.
 *
 * ILIAS is licensed with the GPL-3.0,
 * see https://www.gnu.org/licenses/gpl-3.0.en.html
 * You should have received a copy of said license along with the
 * source code, too.
 *
 * If this is not the case or you just want to try ILIAS, you'll find
 * us at:
 * https://www.ilias.de
 * https://github.com/ILIAS-eLearning
 *
 *********************************************************************/

declare(strict_types=1);

namespace PlagScan;

class API
{
    protected string $host = 'https://api.plagscan.com/';
    protected string $version = 'v3';
    protected string $connection;
    protected static ?self $instance = null;

    public static function getInstance(string $host, string $version = 'v3'): self
    {
        if (self::$instance === null) {
            self::$instance = new self($host, $version);
        }
        return self::$instance;
    }

    public function __construct(string $host, string $version)
    {
        $this->host = $host;
        $this->version = $version;
        $this->connection = $this->host . $this->version;
    }

    /**
     * @return array|string
     */
    public function request(string $end_point, string $request_type, string $access_token = '', array $data = [])
    {
        $ch = curl_init();
        $url = $this->connection . '/' . $end_point;
        if ($access_token) {
            $url .= '?access_token=' . $access_token;
        }
        if ($request_type === 'GET' || $request_type === 'PATCH') {
            foreach ($data as $key => $value) {
                $url .= '&' . $key . '=' . $value;
            }
            if (strpos($url, '?') === false) {
                $url = preg_replace('/\&/', '?', $url, 1);
            }
            $data = [];
        }
        $curlopt = [
            CURLOPT_URL => $url,
            CURLOPT_RETURNTRANSFER => 1,
            CURLOPT_CUSTOMREQUEST => $request_type,
        ];
        if (!empty($data)) {
            if (isset($data['fileUpload'])) {
                $curlopt = $this->createFileDataParams($data) + $curlopt;
            } else {
                $curlopt = [CURLOPT_POSTFIELDS => $data] + $curlopt;
            }
        }
        curl_setopt_array($ch, $curlopt);
        $response = curl_exec($ch);
        curl_close($ch);
        if ($response !== '') {
            $response = json_decode($response, true, 512, JSON_THROW_ON_ERROR);
        }
        if (isset($response['data'])) {
            return $response['data'];
        }
        if (isset($response['error'])) {
            return $response['error']['message'];
        }
        return $response;
    }

    private function createFileDataParams(array $data): array
    {
        $string = '';
        $id = uniqid('', false);
        foreach ($data as $key => $param) {
            if ($key !== 'fileUpload') {
                $string .=
                    "---------------$id\r\n"
                    . "Content-Disposition: form-data; name=\"$key\"\r\n"
                    . "\r\n$param\r\n";
            }
        }
        $fn = explode('/', $data['fileUpload']);
        $fn = end($fn);
        $mime = mime_content_type($data['fileUpload']);
        $content = file_get_contents($data['fileUpload']);
        $string .=
            "---------------$id\r\n"
            . "Content-Disposition: form-data; name=\"fileUpload\"; filename=\"$fn\"\r\n"
            . "Content-Type: $mime\r\n"
            . "Content-Transfer-Encoding: binary\r\n"
            . "\r\n$content\r\n"
            . "---------------$id--\r\n";
        return [
            CURLOPT_POSTFIELDS => $string,
            CURLOPT_HTTPHEADER => [
                "Content-Type: multipart/form-data; boundary=-------------$id",
                'Content-Length: ' . strlen($string)
            ]
        ];
    }
}
